unit Main_U;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  ComCtrls, StdCtrls,
  OTFEUnified_U;

type
  TMain_F = class(TForm)
    reReport: TRichEdit;
    pbSearch: TButton;
    pbProgress: TProgressBar;
    pbCancel: TButton;
    pbPrint: TButton;
    pbClose: TButton;
    PrintDialog: TPrintDialog;
    lblVersion: TLabel;
    procedure pbCancelClick(Sender: TObject);
    procedure pbSearchClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure FormCloseQuery(Sender: TObject; var CanClose: Boolean);
    procedure pbPrintClick(Sender: TObject);
    procedure pbCloseClick(Sender: TObject);
    procedure FormShow(Sender: TObject);
  private
    knownSigs: array of string;
    knownSigsLen: array of integer;
    knownSigsExten: array of string;
    OTFEUnified: TOTFEUnified;
    procedure Search();
    procedure ReportHeader(rootDir: string; includeSubDirs: boolean);
    procedure ReportTail(filesChecked: integer; filesSkipped: integer; found: integer; suspect: integer);
    function  CheckSuspect(filename: string; var isSuspect: integer): boolean;
    function  GetFileExtension(filename: string): string;
  public
    { Public declarations }
  end;

const
  TITLE = 'OTFE Volume File Finder';
  SUSPICIOUS_SIZED_FILE = 31457280;  //  Threshold filesize - 31457280 = 30MB

var
  Main_F: TMain_F;

implementation

{$R *.DFM}

uses SDUGeneral, SDUFileIterator_U,
     FileCtrl,
     OTFEE4M_U,
     OTFEScramDisk_U;

procedure TMain_F.Search();
var
  rootDir: string;
  currFilename: string;
  currSysType: TOTFESystem;
  fileIterator: TSDUFileIterator;
  found: integer;
  suspect: integer;
  filesChecked: integer;
  filesSkipped: integer;
  isSuspect: integer;
begin
  if SelectDirectory('Select directory to search from', '', rootDir) then
    begin
    pbProgress.Min := 0;
    pbProgress.Position := 0;
    pbCancel.tag := 0;
    found := 0;
    suspect:=0;
    filesChecked:=0;
    filesSkipped:=0;
    isSuspect:=-1;

    fileIterator := TSDUFileIterator.Create(nil);
    try
      fileIterator.Directory := rootDir;
      fileIterator.FileMask := '*.*';

      fileIterator.IncludeSubDirs := (MessageDlg('Do you wish to search all subdirectories as well?', mtConfirmation, [mbYes, mbNo], 0)=mrYes);

      pbCancel.enabled := TRUE;
      pbSearch.enabled := FALSE;
      pbPrint.enabled := FALSE;

      ReportHeader(rootDir, fileIterator.IncludeSubDirs);

      fileIterator.Reset();
      pbProgress.Max := fileIterator.Count();
      fileIterator.Reset();

      TOTFEE4M(OTFEUnified.OTFComponents[otfesE4M]).SupportScramDisk := FALSE;
      TOTFEScramDisk(OTFEUnified.OTFComponents[otfesScramDisk]).Active := FALSE;

      currFilename := fileIterator.Next();
      while (currFilename<>'') AND (pbCancel.tag<>1) do
        begin
        if OTFEUnified.TypeOfEncryptedVolFile(currFilename, currSysType) then
          begin
          inc(filesChecked);
          inc(found);
          reReport.lines.add('('+OTFESDispNames[currSysType]+') '+currFilename);
          end
        else if CheckSuspect(currFilename, isSuspect) then
          begin
          inc(filesChecked);
          if isSuspect=2 then
            begin
            inc(found);
            reReport.lines.add('(ScramDisk) '+currFilename);
            end
          else if isSuspect=4 then
            begin
            inc(suspect);
            reReport.lines.add('(Suspicious filesize) '+currFilename);
            end
          else if isSuspect=3 then
            begin
            inc(suspect);
            reReport.lines.add('(Suspicious) '+currFilename);
            end;
          end
        else
          begin
          inc(filesSkipped);
          reReport.lines.add('(Skipped) '+currFilename);
          end;

        pbProgress.position := pbProgress.position +1;
        Application.ProcessMessages();

        currFilename := fileIterator.Next();
        end;

    finally
      fileIterator.Free();
    end;

    ReportTail(filesChecked, filesSkipped, found, suspect);

    pbProgress.position := 0;

    pbCancel.enabled := FALSE;
    pbSearch.enabled := TRUE;
    pbPrint.enabled := TRUE;
    end;

end;

procedure TMain_F.pbCancelClick(Sender: TObject);
begin
  pbCancel.enabled := FALSE;
  pbCancel.tag := 1;

end;

procedure TMain_F.pbSearchClick(Sender: TObject);
begin
  Search();

end;

procedure TMain_F.FormCreate(Sender: TObject);
var
  OTFSystemLoop: TOTFESystem;
begin
  OTFEUnified := TOTFEUnified.Create(nil);
  for OTFSystemLoop:=low(OTFEUnified.OTFEnabledComponents) to high(OTFEUnified.OTFEnabledComponents) do
    begin
    OTFEUnified.OTFEnabledComponents[OTFSystemLoop] := TRUE;
    end;

  OTFEUnified.Active := TRUE;

  self.caption := TITLE;

  SetLength(knownSigs,      6);
  SetLength(knownSigsLen,   6);
  SetLength(knownSigsExten, 6);

  knownSigs[0]      := #$4D + #$5A;
  knownSigsLen[0]   := 2;
  knownSigsExten[0] := 'DLL';

  knownSigs[1]      := #$4D + #$5A;
  knownSigsLen[1]   := 2;
  knownSigsExten[1] := 'EXE';

  knownSigs[2]      := 'RIFF';
  knownSigsLen[2]   := 4;
  knownSigsExten[2] := 'AVI';

  knownSigs[3]      := 'RIFF';
  knownSigsLen[3]   := 4;
  knownSigsExten[3] := 'WAV';

  knownSigs[4]      := 'PK';
  knownSigsLen[4]   := 2;
  knownSigsExten[4] := 'ZIP';

  knownSigs[5]      := 'Rar';
  knownSigsLen[5]   := 3;
  knownSigsExten[5] := 'RAR';

end;

procedure TMain_F.FormCloseQuery(Sender: TObject; var CanClose: Boolean);
begin
  OTFEUnified.Free();

end;

procedure TMain_F.pbPrintClick(Sender: TObject);
begin
  if PrintDialog.Execute() then
    begin
    reReport.Print(TITLE);
    end;

end;

procedure TMain_F.pbCloseClick(Sender: TObject);
begin
  Close();

end;

procedure TMain_F.ReportHeader(rootDir: string; includeSubDirs: boolean);
var
  line: string;
  OTFSystemLoop: TOTFESystem;
begin
  reReport.lines.Clear();
  reReport.SelAttributes.style := [fsUnderline];
  reReport.lines.add(TITLE);
  reReport.SelAttributes.style := [];
  reReport.lines.add('');
  line := 'Searching from: '+rootDir;
  if includeSubDirs then
    begin
    line := line + ' (and all subdirectories)';
    end;

  reReport.lines.add(line);
  reReport.lines.add('Date/Time: '+datetimetostr(now));

  line := 'OTFE volume files searched for: ';
  for OTFSystemLoop:=low(OTFEUnified.OTFEnabledComponents) to high(OTFEUnified.OTFEnabledComponents) do
    begin
    if OTFEUnified.OTFEnabledComponents[OTFSystemLoop]=TRUE then
      begin
      if low(OTFEUnified.OTFEnabledComponents)<>OTFSystemLoop then
        begin
        line := line + ', ';
        end;
      line := line + OTFESDispNames[OTFSystemLoop];
      end;
    end;
  reReport.lines.add(line);
  reReport.lines.add('');

end;

procedure TMain_F.ReportTail(filesChecked: integer; filesSkipped: integer; found: integer; suspect: integer);
begin
  reReport.lines.add('');

  if pbCancel.tag=1 then
    begin
    reReport.lines.add('Search cancelled by user.');
    reReport.lines.add('');
    end;

  reReport.lines.add('Files scanned: '+inttostr(filesChecked));
  reReport.lines.add('Files skipped: '+inttostr(filesSkipped));
  reReport.lines.add('Files appearing to be encrypted volume files: '+inttostr(found));
  reReport.lines.add('Suspicous files: '+inttostr(suspect));
end;


// Returns TRUE if the file was checked, otherwise FALSE
// Sets isSuspect to:
//    -1 if nothing suspected
//    2 if ScramDisk suspected
//    3 if just looks suspicious
//    4 if suspicious filesize
function TMain_F.CheckSuspect(filename: string; var isSuspect: integer): boolean;
var
  buffer: array [1..6656] of byte; // to store the first 6656 bytes of the file
  tmpFile: TFileStream;
  lowDWORD: DWORD;
  highDWORD: DWORD;
  currFileExten: string;
  i: integer;
  j: integer;
  bytesReadIn: integer;
begin
  Result := FALSE;
  isSuspect := -1;

  filename := uppercase(filename);

  try
    tmpFile := TFileStream.Create(filename, fmOpenRead OR fmShareDenyNone);
    try
      lowDWORD := GetFileSize(tmpFile.Handle, @highDWORD);

      tmpFile.Seek(0, soFromBeginning);
      bytesReadIn:=tmpFile.Read(buffer[1], sizeof(buffer));
    finally
      tmpFile.Free();
    end;
  except
    on E:EFOpenError do
      begin
      exit;
      end;
  end;

  Result := TRUE;

  // Check to see if it's looks like a ScramDisk volume with duplicated info
  // (i.e. a v2.02h ScramDisk volume, which has never had it's passwords changed)
  if (isSuspect=-1) then
    begin
    if bytesReadIn=sizeof(buffer) then
      begin
      isSuspect := 2;
      for i:=1 to 420 do
        begin
        if buffer[i]<>buffer[i+6144] then
          begin
          // Nope, not suspicious
          isSuspect := -1;
          break;
          end;
        end;
      end;
    end;

  if isSuspect=-1 then
    begin
    currFileExten := uppercase(GetFileExtension(filename));

    for i:=low(knownSigsExten) to high(knownSigsExten) do
      begin
      if (knownSigsExten[i]=currFileExten) then
        begin
        for j:=1 to knownSigsLen[i] do
          begin
          if char(buffer[j])<>knownSigs[i][j] then
            begin
            isSuspect := 3;
            break;
            end;
          end;

        break;
        end;

      end;
    end;

  // If file length>30MB, and we don't suspect anything so far, then
  // it's automatically suspect
  if isSuspect=-1 then
    begin
    if (highDWORD>0) OR
      (lowDWORD>SUSPICIOUS_SIZED_FILE) then
      begin
      isSuspect := 4;
      end;
    end;

end;

function TMain_F.GetFileExtension(filename: string): string;
var
  i: integer;
  dotPos: integer;
begin
  Result := '';
  dotPos := -1;

  filename := ExtractFileName(filename);
  for i:=length(filename) downto 1 do
    begin
    if filename[i]='.' then
      begin
      dotPos := i;
      break;
      end;
    end;

  if dotPos<>-1 then
    begin
    Result := copy(filename, dotPos+1, length(filename)-dotPos);
    end;

end;

procedure TMain_F.FormShow(Sender: TObject);
var
  majorVersion, minorVersion, releaseVersion, buildVersion: integer;
begin
  SDUGetVersionInfo('', majorVersion, minorVersion, releaseVersion, buildVersion);
  lblVersion.caption := Format('v%d.%.2d.%.2d.%.4d', [majorVersion, minorVersion, releaseVersion, buildVersion]);

end;

END.

